table_name_company_configurations = 'configurations'
table_name_exceptions = 'sessions_exceptions'
table_name_messages = 'messages'
table_name_timeout = 'timeout'

table_prefix_conversational_flow = 'conversational_flow_'
table_prefix_responses = 'responses_'
table_prefix_sessions = 'sessions_'
table_prefix_logs = 'logs_'
table_prefix_logs_end_conversation = 'logs_endconv_'

flow_csv_multiple_response_ids_join = '&'
flow_csv_subflow_identifier = 'subflow'
flow_csv_subflow_delimiter = '|'

void_response = {'RV': '/'}
max_subflow_iterations = 5
